/*
   2001.11.14
   2001.10.11 Modified by Shao-Heng Chang
   
   Improve incoming buffer CRLFmode processing
   Improve IdleTime232 setting method
   support none CRLF mode of 232 port: ModeCrLf=4; as default setting of all 232 port
   
   7522.C
   Demo program for 7522 as an Addressable RS-485 to RS-232 converter.
   using COM1 as RS-232    --> RS-232 Address=A
	 COM2 as RS-485
	 COM3 as RS-232(2) --> RS-232 Address=A+1
   convert non-addressable RS-232 to addressable RS-485

   *** Please use 7188xs.lib Ver 1.16 or latter ****
   Setting COMMAND:
    (1) $AAAn[addr]	--> Read/Set Module address
    (2) $AABn[baudrate] --> Read/Set RS-485(n=0) RS-232(n=1) baudrate
    (3) $AADn[data bit] --> Read/Set Data bit
    (4) $AAPn[parity]	--> Read/Set Parity
    (5) $AAOn[parity]	--> Read/Set stOp bit
    (6) $AA6(ID)	--> Set ID-string
    (7) $AA7		--> Read ID-string
    (8) $AAC[delimiter] --> Read/Set Delimiter
    (9) $AAD		--> Read Delimiter
    (10)(delimiter)AA[by pass data] --> bypass comand-string to COM1/3/4
    (11)$AAKn[CRLF]	--> Read/Set checksum-status
    (12)$AATn[CRLF]	--> Read/Set the end-char-mode of command string
    (13)$AAW		--> Read CTS of COM-1/3
    (14)$AAXV		--> Set  RTS of COM-1/3
    (15)$AAYN		--> Read OnBoard DI-1/2/3
    (16)$AAZNV		--> Set  OnBoard DO-1/2/3
    (17)#**		--> Synchronized Sampling
    (18)$AA4		--> Read the synchronized data
    (19)$AA5		--> Read the reset-status
    (20)$AAM		--> Read Module name
    (21)$AAF		--> Read Firmwre Revsion Number
    (22)~**		--> Host is OK
    (23)~AA0		--> Read  the module Status
    (24)~AA1		--> Reset the module Status
    (25)~AA2		--> Reset the host watchdog status & timer
    (26)~AA3ETT 	--> Enable/Disable host watchdog timer
    (27)~AA4P/S 	--> Read PowerOn/Safe value
    (28)~AA5P/S 	--> Set  PowerOn/Safe value

   EEPROM data use block 7
   addr 0,1: save 7522 as a flag
	2  : RS-485 baudrate(1-10)
	3  : RS-485 parity(0,1,2)for(N,E,O)
	4  : RS-485 data bits (7,8)
	5  : RS-485 stop bit (always 1)
	6  : RS-232 baudrate(1-10)for(300-115200)
	7  : RS-232 parity(0,1,2)for(N,E,O)
	8  : RS-232 data bits (7,8)
	9  : RS-232 stop bit (always 1)
     **** 10-17 for 7522/7523
	10 : RS-232(2) baudrate(1-10)for(300-115200)
	11 : RS-232(2) parity(0,1,2)for(N,E,O)
	12 : RS-232(2) data bits (7,8)
	13 : RS-232(2) stop bit (always 1)
	14 : RS-232(3) baudrate(1-10)for(300-115200)
	15 : RS-232(3) parity(0,1,2)for(N,E,O)
	16 : RS-232(3) data bits (7,8)
	17 : RS-232(3) stop bit (always 1)
    ****
	20 : ChechSum(0:disable, 1:enable)
	21 : HostWdtEnable
	22 : HostWdtTimer
	23 : HostWdtPowerOnValue
	24 : HostWdtPowerSafeValue
    ****
	31 : delimiter
	32 : delimiter(2)
	33 : delimiter(3)
    ****
	40 : RS-485 CRLF mode
	41 : RS-232 CRLF mode
	42 : RS-232(2) CRLF mode
	43 : RS-232(3) CRLF mode
    ****
	50 : address
    ****
	100-149: ID for RS-232
	150-199: ID for RS-232(2)
	200-249: ID for RS-232(3)

Release Note:
[June/2000]
  1. User must send command to COM2(RS-485), if the command is used to
     set configuration of 7522, the 7522 will directly echo message to COM2.
     If the command is used to	bypass data to RS-232 device, the data will be
     sent to COM1/3(RS-232). Any message come from COM1/3(RS-232) will be
     echo to COM2(RS-485) at any time.
  2. The COM port address of 7521 is given as following:
     COM1=RS-232    --> RS-232 Address=AA
     COM2=RS-485
  3. The COM port address of 7522 is given as following:
     COM1=RS-232    --> RS-232 Address=AA
     COM2=RS-485
     COM3=RS-232    --> RS-232 Address=AA+1
     7522 is used as two   7521 with address AA, AA+1.	      And the two
     COM port must use the same ModeCrLf485 and the ModeCrLf232 can be different.
     The delimiters for these three COM port can be different.
  4. comparision table of COM1, COM3 & COM4 of 7522/7523
			COM1	       COM3&COM4
     ---------------------------------------------------
     start-bit		1	       1
     data-bit		7/8	       7/8
     parity-bit 	No/Even/Odd    No/Even/Odd
     stop-bit		1	       1/2
     baud rate		115.2K max.    115.2K max.
     interrupt		Yes	       Yes
     FIFO		double buffer  16 bytes
     connect HP34401A	No	       Yes    (HP34401A require two stop-bit)

     So 7521 can not link to HP34401A.
     7522 can link to one HP34401A.
     7523 can link to two HP34401A.
*/

/* ------------------------------------------------------------- */

#include<stdio.h>
#include<stdlib.h>
#include"7188xb.h"

#define InstallCom232 InstallCom1
#define IsCom232      IsCom1
#define ReadCom232    ReadCom1
#define ToCom232      ToCom1
#define ToCom232Str   ToCom1Str
#define ToComBufn232  ToCom1Bufn
#define RestoreCom232 RestoreCom1

#define InstallCom485 InstallCom2
#define IsCom485      IsCom2
#define ReadCom485    ReadCom2
#define ToCom485      ToCom2
#define ToCom485Str   ToCom2Str
#define ToComBufn485  ToCom2Bufn
#define RestoreCom485 RestoreCom2
#define WaitTransmitOver485 WaitTransmitOver2
/*
#define InstallCom_3 InstallCom3
#define IsCom_3	IsCom3
#define ReadCom_3	ReadCom3
#define ToCom_3	ToCom3
#define ToComStr_3	ToCom3Str
#define ToComBufn_3	ToCom3Bufn
#define RestoreCom_3 RestoreCom3
*/
#define dPrint	   Print
#define Putchd	   Putch
#define ReadComd   Getch
#define IsComd	   Kbhit

#define IDLETOOLONG 10000
int IdleTime485;
int IdleTime232;
int IdleTime_3;
int IdleTime_4;
int IdleTime_5;
int IdleTime_6;
int IdleTime_7;
int IdleTime_8;

int DEBUG_7527=0;

int WaitToDo=0;
int NewCheckSum,NewDataBit,NewParity;
unsigned long NewBaudrate;
enum WaitToDoMode {
 _ChangeNothing=0,
 _ChangeCheckSum,
 _ChangeBaudrate,
 _ChangeDataBit,
 _ChangeParity,
};

int quit=0;
long BAUD[11]={
9600L,
300L,
600L,
1200L,
2400L,
4800L,
9600L,
19200L,
38400L,
57600L,
115200L
};

int Baud485=6;
int Parity485=0;
int Data485=8;
int Stop485=1;
int CheckSum=0;

int Baud232=6;
int Parity232=0;
int Data232=8;
int Stop232=1;

int Baud_3=6;
int Parity_3=0;
int Data_3=8;
int Stop_3=1;

int Baud_4=6;
int Parity_4=0;
int Data_4=8;
int Stop_4=1;

int Baud_5=6;
int Parity_5=0;
int Data_5=8;
int Stop_5=1;

int Baud_6=6;
int Parity_6=0;
int Data_6=8;
int Stop_6=1;

int Baud_7=6;
int Parity_7=0;
int Data_7=8;
int Stop_7=1;

int Baud_8=6;
int Parity_8=0;
int Data_8=8;
int Stop_8=1;

unsigned char Delimiter=':';
unsigned char Delimiter_3=':';
unsigned char Delimiter_4=':';
unsigned char Delimiter_5=':';
unsigned char Delimiter_6=':';
unsigned char Delimiter_7=':';
unsigned char Delimiter_8=':';

int Address=1;
//int Address;

int ModeCrLf485=4;
int ModeCrLf232=4;
int ModeCrLf_3=4;
int ModeCrLf_4=4;
int ModeCrLf_5=4;
int ModeCrLf_6=4;
int ModeCrLf_7=4;
int ModeCrLf_8=4;
//int Mode4End=13;

int CmdMode=0;

char ID[50]= "ICPDAS-7527-2";   /* ID-string for COM2(RS485)    */
char ID2[50]="ICPDAS-7527-1";   /* ID-string for COM1(RS232)    */
char ID3[50]="ICPDAS-7527-3";   /* ID-string for COM3(RS232)    */
char ID4[50]="ICPDAS-7527-4";   /* ID-string for COM4(RS232)    */
char ID5[50]="ICPDAS-7527-5";   /* ID-string for COM5(RS232)    */
char ID6[50]="ICPDAS-7527-6";   /* ID-string for COM6(RS232)    */
char ID7[50]="ICPDAS-7527-7";   /* ID-string for COM7(RS232)    */
char ID8[50]="ICPDAS-7527-8";   /* ID-string for COM8(RS232)    */

unsigned long far * timetick=(unsigned long far *)0x0040006CL;
char ResetStatus=1,SyncData=0, FirstRead=0;
char HostWdtEnable,HostWdtModuleStatus=0,HostWdtTimer;
char HostWdtPowerOnValue,HostWdtSafeValue;
unsigned long HostTimerStart;

void SetAllDo(char);

/* -------------------------------------------------------------------- */

void Open7527(void)
{ int Is7527;
  unsigned char *dataptr=(unsigned char *)&Is7527;
  int addr;

/* step 1: read/write the configuration data from EEPROM	*/
/* if first-time RUN --> write INIT-DATA to EEPROM		*/
/* if not-first-time RUN --> load configuration from EEPROM	*/

  dataptr[0]=ReadEEP(7,0);	/* ID, high byte		*/
  dataptr[1]=ReadEEP(7,1);	/* ID, low  byte		*/

  if(Is7527==7527){/* this is not-first-time RUN		*/
		   /* --> load configuration data from EEPROM	*/
     Baud485=ReadEEP(7,2);	/* baud rate  of COM2		*/
     Parity485=ReadEEP(7,3);	/* Parity-bit of COM2		*/
     Data485=ReadEEP(7,4);	/* data-bit   of COM2		*/
     Stop485=ReadEEP(7,5);	/* stop-bit   of COM2		*/

     Baud232=ReadEEP(7,6);	/* baud rate  of COM1		*/
     Parity232=ReadEEP(7,7);	/* Parity-bit of COM1		*/
     Data232=ReadEEP(7,8);	/* data-bit   of COM1		*/
     Stop232=ReadEEP(7,9);	/* stop-bit   of COM1		*/

     Baud_3=ReadEEP(7,10);	/* baud rate  of COM3		*/
     Parity_3=ReadEEP(7,11);    /* Parity-bit of COM3		*/
     Data_3=ReadEEP(7,12);	/* data-bit   of COM3		*/
     Stop_3=ReadEEP(7,13);	/* stop-bit   of COM3		*/

     Baud_4=ReadEEP(7,51);	/* baud rate  of COM4		*/
     Parity_4=ReadEEP(7,52);    /* Parity-bit of COM4		*/
     Data_4=ReadEEP(7,53);	/* data-bit   of COM4		*/
     Stop_4=ReadEEP(7,54);	/* stop-bit   of COM4		*/

     Baud_5=ReadEEP(7,55);	/* baud rate  of COM5		*/
     Parity_5=ReadEEP(7,56);    /* Parity-bit of COM5		*/
     Data_5=ReadEEP(7,57);	/* data-bit   of COM5		*/
     Stop_5=ReadEEP(7,58);	/* stop-bit   of COM5		*/

     Baud_6=ReadEEP(7,62);	/* baud rate  of COM6		*/
     Parity_6=ReadEEP(7,63);    /* Parity-bit of COM6		*/
     Data_6=ReadEEP(7,64);	/* data-bit   of COM6		*/
     Stop_6=ReadEEP(7,65);	/* stop-bit   of COM6		*/

     Baud_7=ReadEEP(7,66);	/* baud rate  of COM7		*/
     Parity_7=ReadEEP(7,67);    /* Parity-bit of COM7		*/
     Data_7=ReadEEP(7,68);	/* data-bit   of COM7	*/
     Stop_7=ReadEEP(7,69);	/* stop-bit   of COM7		*/

     Baud_8=ReadEEP(7,70);	/* baud rate  of COM8		*/
     Parity_8=ReadEEP(7,71);    /* Parity-bit of COM8		*/
     Data_8=ReadEEP(7,72);	/* data-bit   of COM8		*/
     Stop_8=ReadEEP(7,73);	/* stop-bit   of COM8		*/

     Delimiter=ReadEEP(7,31);	/* delimiter  of COM1		*/
     Delimiter_3=ReadEEP(7,32); /* delimiter  of COM3		*/
     Delimiter_4=ReadEEP(7,33); /* delimiter  of COM4		*/
     Delimiter_5=ReadEEP(7,34); /* delimiter  of COM5		*/
     Delimiter_6=ReadEEP(7,35); /* delimiter  of COM6		*/
     Delimiter_7=ReadEEP(7,36); /* delimiter  of COM7		*/
     Delimiter_8=ReadEEP(7,37); /* delimiter  of COM8		*/

     CheckSum=ReadEEP(7,20);	/* checksum enable/disable	*/
     HostWdtEnable=ReadEEP(7,21);/* HostWDT enable/disable	*/
     if (HostWdtEnable) 	/* enable HostWDT		*/
	{
	_asm cli
	HostTimerStart=*timetick;
	_asm sti
	}
     HostWdtTimer=ReadEEP(7,22);/* timer for HostWDT		*/

     HostWdtPowerOnValue=ReadEEP(7,23); /* PowerOn-value for	*/
     SetAllDo(HostWdtPowerOnValue);	/* OnBoard DO-1/2/3	*/

     /* if (HostWDT is enable & Host-is-failure 		*/
     /* all OnBoard-DO wil go to their safe-value immediately	*/
     HostWdtSafeValue=ReadEEP(7,24);	/* Safe-value for	*/
					/* OnBoard DO-1/2/3	*/

     Address=ReadEEP(7,50);	 /* COM1=AA,COM3=AA+1,COM4=AA+2 */
     ModeCrLf485=ReadEEP(7,40);  /* end-char mode of COM2	*/
     ModeCrLf232=ReadEEP(7,41);  /* end-char mode of COM1	*/
     ModeCrLf_3=ReadEEP(7,42);/* end-char mode of COM3	*/
     ModeCrLf_4=ReadEEP(7,43);/* end-char mode of COM4	*/
     ModeCrLf_5=ReadEEP(7,44);/* end-char mode of COM5	*/
     ModeCrLf_6=ReadEEP(7,45);/* end-char mode of COM6	*/
     ModeCrLf_7=ReadEEP(7,46);/* end-char mode of COM7	*/
     ModeCrLf_8=ReadEEP(7,47);/* end-char mode of COM8	*/
  }
  else{ /* this is first-time run --> write INIT-DATA to EEPROM */
    EnableEEP();
    Is7527=7527;
    WriteEEP(7,0,dataptr[0]);
    WriteEEP(7,1,dataptr[1]);
    WriteEEP(7,2,6); /* 9600	     */     /* COM2=RS-485 */
    WriteEEP(7,3,0); /* No parity    */
    WriteEEP(7,4,8); /* 8 data bits  */
    WriteEEP(7,5,1); /* 1 stop bit   */

    WriteEEP(7,6,6); /* 9600	     */     /* COM1=RS-232 */
    WriteEEP(7,7,0); /* No parity    */
    WriteEEP(7,8,8); /* 8 data bits  */
    WriteEEP(7,9,1); /* 1 stop bit   */

    WriteEEP(7,10,6); /* 9600	     */     /* COM3=RS-232 */
    WriteEEP(7,11,0); /* No parity   */
    WriteEEP(7,12,8); /* 8 data bits */
    WriteEEP(7,13,1); /* 1 stop bit  */

    WriteEEP(7,51,6); /* 9600	     */     /* COM4=RS-232 */
    WriteEEP(7,52,0); /* No parity    */
    WriteEEP(7,53,8); /* 8 data bits  */
    WriteEEP(7,54,1); /* 1 stop bit   */

    WriteEEP(7,55,6); /* 9600	     */     /* COM5=RS-232 */
    WriteEEP(7,56,0); /* No parity   */
    WriteEEP(7,57,8); /* 8 data bits */
    WriteEEP(7,58,1); /* 1 stop bit  */

    WriteEEP(7,62,6); /* 9600	     */     /* COM6=RS-485 */
    WriteEEP(7,63,0); /* No parity    */
    WriteEEP(7,64,8); /* 8 data bits  */
    WriteEEP(7,65,1); /* 1 stop bit   */

    WriteEEP(7,66,6); /* 9600	     */     /* COM7=RS-232 */
    WriteEEP(7,67,0); /* No parity    */
    WriteEEP(7,68,8); /* 8 data bits  */
    WriteEEP(7,69,1); /* 1 stop bit   */

    WriteEEP(7,70,6); /* 9600	     */     /* COM8=RS-232 */
    WriteEEP(7,71,0); /* No parity   */
    WriteEEP(7,72,8); /* 8 data bits */
    WriteEEP(7,73,1); /* 1 stop bit  */

    WriteEEP(7,20,CheckSum); /* checksum = 0 = disable	*/
    WriteEEP(7,21,0); /* HostWdtEnable = disable  */
    WriteEEP(7,22,0); /* HostWdtTimer  = 0 ms	  */
    WriteEEP(7,23,0); /* HostWdtPowerOnValue =OFF */
    WriteEEP(7,24,0); /* HostWdtSafeValue    =OFF */

    WriteEEP(7,31,':'); /* delimiter=':' for COM1 */
    WriteEEP(7,32,':'); /* delimiter=':' for COM3 */
    WriteEEP(7,33,':'); /* delimiter=':' for COM4 */
    WriteEEP(7,34,':'); /* delimiter=':' for COM5 */
    WriteEEP(7,35,':'); /* delimiter=':' for COM6 */
    WriteEEP(7,36,':'); /* delimiter=':' for COM7 */
    WriteEEP(7,37,':'); /* delimiter=':' for COM8 */

    WriteEEP(7,40,4); /* end-char=0=CR (for COM2) */
    WriteEEP(7,41,4); /* end-char=4=NO (for COM1) */
    WriteEEP(7,42,4); /* end-char=4=NO (for COM3) */
    WriteEEP(7,43,4); /* end-char=4=NO (for COM4) */
    WriteEEP(7,44,4); /* end-char=4=NO (for COM5) */
    WriteEEP(7,45,4); /* end-char=4=NO (for COM6) */
    WriteEEP(7,46,4); /* end-char=4=NO (for COM7) */
    WriteEEP(7,47,4); /* end-char=4=NO (for COM8) */

    WriteEEP(7,50,1); /* Address=1 --> AA=01	  */

    dataptr=ID;       /* ID-string of COM2(RS485) */
    for(addr=100; addr<150 && *dataptr; addr++){
	WriteEEP(7,addr,*dataptr++);
    }
    if(addr<150) WriteEEP(7,addr,0); /* end of ID */

    dataptr=ID2;      /* ID-string of COM1(RS232) */
    for(addr=150; addr<200 && *dataptr; addr++){
	WriteEEP(7,addr,*dataptr++);
    }
    if(addr<200) WriteEEP(7,addr,0); /* end of ID */

    dataptr=ID3;      /* ID-string of COM3(RS232) */
    for(addr=200; addr<250 && *dataptr; addr++){
	WriteEEP(7,addr,*dataptr++);
    }
    if(addr<250) WriteEEP(7,addr,0); /* end of ID */

////
    dataptr=ID4;      /* ID-string of COM4(RS232) */
    for(addr=0; addr<50 && *dataptr; addr++){
	WriteEEP(6,addr,*dataptr++);
    }
    if(addr<50) WriteEEP(6,addr,0); /* end of ID */

    dataptr=ID5;      /* ID-string of COM5(RS232) */
    for(addr=50; addr<100 && *dataptr; addr++){
	WriteEEP(6,addr,*dataptr++);
    }
    if(addr<100) WriteEEP(6,addr,0); /* end of ID */

    dataptr=ID6;      /* ID-string of COM6(RS232) */
    for(addr=100; addr<150 && *dataptr; addr++){
	WriteEEP(6,addr,*dataptr++);
    }
    if(addr<150) WriteEEP(6,addr,0); /* end of ID */

    dataptr=ID7;      /* ID-string of COM7(RS232) */
    for(addr=150; addr<200 && *dataptr; addr++){
	WriteEEP(6,addr,*dataptr++);
    }
    if(addr<200) WriteEEP(6,addr,0); /* end of ID */

    dataptr=ID8;      /* ID-string of COM8(RS232) */
    for(addr=200; addr<250 && *dataptr; addr++){
	WriteEEP(6,addr,*dataptr++);
    }
    if(addr<250) WriteEEP(6,addr,0); /* end of ID */

    ProtectEEP();
  }

/* step 2: check the init* pin is floating or connected to GND	     */

  if(ReadInitPin()){	/* Init* connect to GND --> goto INIT* state */
     Address=0;	/* AA=00				     */
     ModeCrLf485=4;	/* end-char-mode of COM2		     */
     ModeCrLf232=4;	/* end-char-mode of COM1		     */
     ModeCrLf_3=4;/* end-char-mode of COM3		     */
     ModeCrLf_4=4;/* end-char-mode of COM4		     */
     ModeCrLf_5=4;/* end-char-mode of COM5		     */
     ModeCrLf_6=4;/* end-char-mode of COM6		     */
     ModeCrLf_7=4;/* end-char-mode of COM7		     */
     ModeCrLf_8=4;/* end-char-mode of COM8		     */

     if(!DEBUG_7527)InstallCom232(BAUD[6],8,0); /* COM1, 9600, N81   */

     Baud485=6; 			/* short-code for 9600 is 6  */
     InstallCom485(BAUD[6],8,0);	/* COM2, RS-485, 9600, N81   */

     InstallCom_36(BAUD[6],8,0,1);	/* COM6, RS-232, 9600, N81   */
     InstallCom_47(BAUD[6],8,0,1);	/* COM7, RS-232, 9600, N81   */
     InstallCom_58(BAUD[6],8,0,1);	/* COM8, RS-232, 9600, N81   */
     SetBaudrate_3(BAUD[6]);
     SetDataFormat_3(8,0,1);
     SetBaudrate_4(BAUD[6]);
     SetDataFormat_4(8,0,1);
     SetBaudrate_5(BAUD[6]);
     SetDataFormat_5(8,0,1);
     CheckSum=0;
  }
  else {		/* Init* float --> state saved in EEPROM     */
		/* load AA from EEPROM	     */
     Address=ReadEEP(7,50);
					/* initiate COM1/2/3/4	     */
     if(!DEBUG_7527)InstallCom232(BAUD[Baud232],Data232,Parity232);
     InstallCom485(BAUD[Baud485],Data485,Parity485);
     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
     SetBaudrate_3(BAUD[Baud_3]);
     SetDataFormat_3(Data_3,Parity_3,Stop_3);
     SetBaudrate_4(BAUD[Baud_4]);
     SetDataFormat_4(Data_4,Parity_4,Stop_4);
     SetBaudrate_5(BAUD[Baud_5]);
     SetDataFormat_5(Data_5,Parity_5,Stop_5);

  }

  if(DEBUG_7527){
     dPrint("Address=%02X Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d CheckSum=%d\n\r",
		Address,BAUD[Baud485],Data485,Parity485,Stop485,ModeCrLf485,CheckSum);
     dPrint("232: Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud232],Data232,Parity232,Stop232,ModeCrLf232);
     dPrint("232(2): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_3],Data_3,Parity_3,Stop_3,ModeCrLf_3);
     dPrint("232(COM4): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_4],Data_4,Parity_4,Stop_4,ModeCrLf_4);
     dPrint("232(COM5): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_5],Data_5,Parity_5,Stop_5,ModeCrLf_5);
     dPrint("232(COM6): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_6],Data_6,Parity_6,Stop_6,ModeCrLf_6);
     dPrint("232(COM7): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_7],Data_7,Parity_7,Stop_7,ModeCrLf_7);
     dPrint("232(COM8): Baud=%ld Data=%d Parity=%d Stop=%d CRmode=%d\n\r",
		BAUD[Baud_8],Data_8,Parity_8,Stop_8,ModeCrLf_8);
  }

  IdleTime485=1+(int)(11000.0/(float)BAUD[Baud485]);
  IdleTime232=1+(int)(11000.0/(float)BAUD[Baud232]);
  IdleTime_3=1+(int)(11000.0/(float)BAUD[Baud_3]);
  IdleTime_4=1+(int)(11000.0/(float)BAUD[Baud_4]);
  IdleTime_5=1+(int)(11000.0/(float)BAUD[Baud_5]);
  IdleTime_6=1+(int)(11000.0/(float)BAUD[Baud_6]);
  IdleTime_7=1+(int)(11000.0/(float)BAUD[Baud_7]);
  IdleTime_8=1+(int)(11000.0/(float)BAUD[Baud_8]);

/* step 3: show address & baudrate of COM2 in 7-SEG LEDs       */

  Init5DigitLed();			  /* init 7-SEG LEDs   */
  Show5DigitLed(1,Address>>4);	  /* high nibble of AA */
  Show5DigitLedWithDot(2,Address&0x0F);/* low  nibble of AA */
  Show5DigitLedWithDot(3,Baud485);	  /* baud rate of COM2 */
  Show5DigitLed(4,Baud232);               /* baud rate of COM1 */
  Show5DigitLed(5,16);			  /* for user	       */
}

void Close7527(void)
{
  if(!DEBUG_7527) RestoreCom232();	  /* COM1	       */
  RestoreCom485();			  /* COM2	       */
  RestoreCom_36();			  /* COM3	       */
  RestoreCom_47();			  /* COM4	       */
  RestoreCom_58();			  /* COM5              */

  Show5DigitLed(1,7);
  Show5DigitLed(2,5);
  Show5DigitLed(3,2);
  Show5DigitLedWithDot(4,7);
  Show5DigitLedSeg(5,0x37);
}

/* --- send a command string to RS485 port -------------------- */
int COMMAND=0;
void SendCmdToCom485Str(unsigned char *cmd,int len)
{ unsigned char sum;

/*  step 1: send command-part			   */
    ToComBufn485(cmd,len);             ////////////if the end is not 0x00, this function fails.
    if(DEBUG_7527) dPrint(cmd);

/*  step 2: if checksumis-enable send checksum-part */
/*  note: this part is for COM2(RS485) only	    */

    if(CheckSum){
       int i;
       unsigned char *dat=cmd;
       sum=0;
       for(i=0;i<len;i++){
	 sum+=*dat;
	 dat++;
       }
       ToCom485(hex_to_ascii[sum >> 4]);
       ToCom485(hex_to_ascii[sum & 0x0F]);
       if(DEBUG_7527){
	  Putchd(hex_to_ascii[sum >> 4]);
	  Putchd(hex_to_ascii[sum & 0x0F]);
       }
    }

/*  step 3: send end-char-part			    */
    switch(ModeCrLf485){
      case 0:
	   ToCom485('\r');
	   if(DEBUG_7527) dPrint("Cr\n");   // CrLfMode=0=CR
	   break;
      case 1:
	   ToCom485Str("\r\n");
	   if(DEBUG_7527) dPrint("CrLf\n"); // CrLfMode=1=CR+LF
	   break;
      case 2:
	   ToCom485('\n');
	   if(DEBUG_7527) dPrint("Lf\n");   // CrLfMode=2=LF
	   break;
      case 3:
	   ToCom485Str("\n\r");
	   if(DEBUG_7527) dPrint("LfCr\n"); // CrLfMode=3=LF+CR
	   break;
      case 4:
	   if(COMMAND)
	   {
	      ToCom485Str("\r");
	      COMMAND=0;
	      if(DEBUG_7527) dPrint("Cr\n"); // CrLfMode=1=CR+LF

	   }
    }
}

/* --- if receive a commmad-string from COM1 --> bypass to COM2 -- */
unsigned char RingBuf1[1024];
int InIdx1=0,OutIdx1=0;
int WaitCom1Echo=0;
/* ------------------------------------------------------------- */
void ResetRingBuf1(void)
{
  InIdx1=OutIdx1=0;
}

void AddDataToRingBuf1(unsigned char data)
{
   if(((InIdx1+1)&1023) != OutIdx1){
       RingBuf1[InIdx1++]=data;
       if(InIdx1>=1024) InIdx1=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf1(unsigned char *buf)
{
 //////////
  int i=0;
   unsigned char data;
   while(InIdx1 != OutIdx1)
   {
      *buf=(data=RingBuf1[OutIdx1]);
      buf++;
      i++;
      OutIdx1++; if(OutIdx1>=1024) OutIdx1=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf232==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf232==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf232==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf232==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
   return i;
}

/* --- if receive a commmad-string from COM1 --> bypass to COM2 -- */

unsigned char Cmd232[256];
//unsigned char Echo232[80];
int Idx232=0;
void DoCom232(void) /* COM1=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom232()){
    if(WaitCom1Echo){
       data=ReadCom232();
       LastHitTimeTick= *timetick;
       Cmd232[Idx232++]=data;
       if(data=='\r'){
	  if(ModeCrLf232==0){		      /* CrLfMode=0=CR --> match     */
	     Cmd232[Idx232-1]=0;
	     SendCmdToCom485Str(Cmd232,Idx232-1);	      /* send command-string to COM2 */
	     Idx232=0;
	     reply=1;
	  }
	  else if(ModeCrLf232==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(Cmd232[Idx232-2]=='\n'){
		     Cmd232[Idx232-2]=0;
		     SendCmdToCom485Str(Cmd232,Idx232-2); /* send command-string to COM2 */
		     Idx232=0;
		     reply=1;
		  }
	  }
       }
       else if(data=='\n'){
	       if(ModeCrLf232==2){	      /* CrLfMode=2=LF --> match     */
		  Cmd232[Idx232-1]=0;
		  SendCmdToCom485Str(Cmd232,Idx232-1);    /* send command-string to COM2 */
		  Idx232=0;
		  reply=1;
	       }
	       else if(ModeCrLf232==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(Cmd232[Idx232-2]=='\r'){
		      Cmd232[Idx232-2]=0;
		      SendCmdToCom485Str(Cmd232,Idx232-2);/* send command-string to COM2 */
		      Idx232=0;
		      reply=1;
		    }
	       }
       }

    }
    else {
       AddDataToRingBuf1(ReadCom232());
    }
  }

  if( (Idx232!=0) && (ModeCrLf232==4) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime232)
     {
	   int y;
	   SendCmdToCom485Str(Cmd232,Idx232);//0~ Idx232
/*
	   for(y=0;y<Idx232;y++)
	   {
	      if(Cmd232[y]>31)printCom2("[%c]",Cmd232[y]);
	      else printCom2("[%d]",Cmd232[y]);
	   }
*/
	   Idx232=0;
//     printCom1("WaitCom1Echo=%d",WaitCom1Echo);
//       printCom1("||timeout =%lu",LastHitTimeTick);
       reply=1;
//	   LastHitTimeTick= * timetick;
     }

  }
//     printCom1("WaitCom1Echo=%d",WaitCom1Echo);

  if(WaitCom1Echo&&(reply==1))
  {
//     if(LastHitTimeTick==0)
//	printCom1(" _LastHitTimeTick error");
     if( ((*timetick)-LastHitTimeTick)>(IDLETOOLONG+IdleTime232)){
     //unsigned long diff=(*timetick)-LastHitTimeTick;

//	printCom1("\n\rLast=%lu",LastHitTimeTick);
//     if( ((*timetick)-LastHitTimeTick)>5000){
//					    ;
//     printCom1("\n\rdiff=%ld",diff);
//     printCom1("\n\r*timetick=%ld",*timetick);

//     printCom1("WaitCom1Echo=%d",WaitCom1Echo);
	WaitCom1Echo=0;
	reply=0;
     }
//	printCom1("\n\r,%lu",LastHitTimeTick);

  }

//     printCom1("\n\rLastHitTimeTick=%ld",LastHitTimeTick);

}

unsigned char RingBuf_3[1024];
int InIdx_3=0,OutIdx_3=0;
int WaitCom1Echo_3=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_3(void)
{
  InIdx_3=OutIdx_3=0;
}

void AddDataToRingBuf_3(unsigned char data)
{
   if(((InIdx_3+1)&1023) != OutIdx_3){
       RingBuf_3[InIdx_3++]=data;
       if(InIdx_3>=1024) InIdx_3=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_3(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_3 != OutIdx_3)
   {
      *buf=(data=RingBuf_3[OutIdx_3]);
      buf++;
      i++;
      OutIdx_3++; if(OutIdx_3>=1024) OutIdx_3=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_3==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_3==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_3==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_3==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM3 --> bypass to COM2 -- */

unsigned char Cmd_3[256];
//unsigned char Echo_3[80];
int Idx_3=0;
void DoCom_3(void) /* COM3=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_3()){
    if(WaitCom1Echo_3){
      data=ReadCom_3();
      LastHitTimeTick= * timetick;
      Cmd_3[Idx_3++]=data;
      if(data=='\r'){
	if(ModeCrLf_3==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_3[Idx_3-1]=0;
	  SendCmdToCom485Str(Cmd_3,Idx_3-1);        /* send command-string to COM2 */
	  Idx_3=0;
	  reply=1;
	}
	else if(ModeCrLf_3==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_3[Idx_3-2]=='\n'){
		  Cmd_3[Idx_3-2]=0;
		  SendCmdToCom485Str(Cmd_3,Idx_3-2);/* send command-string to COM2 */
		  Idx_3=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_3==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_3[Idx_3-1]=0;
	       SendCmdToCom485Str(Cmd_3,Idx_3-1);   /* send command-string to COM2 */
	       Idx_3=0;
	       reply=1;
	    }
	    else if(ModeCrLf_3==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_3[Idx_3-2]=='\r'){
		   Cmd_3[Idx_3-2]=0;
		   SendCmdToCom485Str(Cmd_3,Idx_3-2);/* send command-string to COM2 */
		   Idx_3=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_3(ReadCom_3());
    }
  }
  if( (ModeCrLf_3==4) &&(Idx_3!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_3)
     {
	   SendCmdToCom485Str(Cmd_3,++Idx_3);
	   Idx_3=0;
	   reply=1;
     }

  }
  if(WaitCom1Echo_3&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IDLETOOLONG+IdleTime_3))
     {
	WaitCom1Echo_3=0;
	reply=0;
     }
  }

}

///////////////////////////////////////////////////////////////////
//COM4
unsigned char RingBuf_4[1024];
int InIdx_4=0,OutIdx_4=0;
int WaitCom1Echo_4=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_4(void)
{
  InIdx_4=OutIdx_4=0;
}

void AddDataToRingBuf_4(unsigned char data)
{
   if(((InIdx_4+1)&1023) != OutIdx_4){
       RingBuf_4[InIdx_4++]=data;
       if(InIdx_4>=1024) InIdx_4=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_4(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_4 != OutIdx_4)
   {
      *buf=(data=RingBuf_4[OutIdx_4]);
      buf++;
      i++;
      OutIdx_4++; if(OutIdx_4>=1024) OutIdx_4=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_4==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_4==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_4==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_4==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM4 --> bypass to COM2 -- */

unsigned char Cmd_4[256];
//unsigned char Echo_4[80];
int Idx_4=0;
void DoCom_4(void) /* COM3=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_4()){
    if(WaitCom1Echo_4){
      data=ReadCom_4();
      LastHitTimeTick= * timetick;
      Cmd_4[Idx_4++]=data;
      if(data=='\r'){
	if(ModeCrLf_4==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_4[Idx_4-1]=0;
	  SendCmdToCom485Str(Cmd_4,Idx_4-1);        /* send command-string to COM2 */
	  Idx_4=0;
	  reply=1;
	}
	else if(ModeCrLf_4==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_4[Idx_4-2]=='\n'){
		  Cmd_4[Idx_4-2]=0;
		  SendCmdToCom485Str(Cmd_4,Idx_4-2);/* send command-string to COM2 */
		  Idx_4=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_4==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_4[Idx_4-1]=0;
	       SendCmdToCom485Str(Cmd_4,Idx_4-1);   /* send command-string to COM2 */
	       Idx_4=0;
	       reply=1;
	    }
	    else if(ModeCrLf_4==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_4[Idx_4-2]=='\r'){
		   Cmd_4[Idx_4-2]=0;
		   SendCmdToCom485Str(Cmd_4,Idx_4-2);/* send command-string to COM2 */
		   Idx_4=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_4(ReadCom_4());
    }
  }
  if( (ModeCrLf_4==4) &&(Idx_4!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_4)
     {
	   SendCmdToCom485Str(Cmd_4,Idx_4);
	   Idx_4=0;
	   reply=1;
     }

  }
  if(WaitCom1Echo_4&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IdleTime_4+IDLETOOLONG) )
     {
	WaitCom1Echo_4=0;
	reply=0;
     }
  }
}

///////////////////////////////////////////////////////////////////
//COM5
unsigned char RingBuf_5[1024];
int InIdx_5=0,OutIdx_5=0;
int WaitCom1Echo_5=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_5(void)
{
  InIdx_5=OutIdx_5=0;
}

void AddDataToRingBuf_5(unsigned char data)
{
   if(((InIdx_5+1)&1023) != OutIdx_5){
       RingBuf_5[InIdx_5++]=data;
       if(InIdx_5>=1024) InIdx_5=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_5(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_5 != OutIdx_5)
   {
      *buf=(data=RingBuf_5[OutIdx_5]);
      buf++;
      i++;
      OutIdx_5++; if(OutIdx_5>=1024) OutIdx_5=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_5==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_5==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_5==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_5==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM5 --> bypass to COM2 -- */

unsigned char Cmd_5[256];
//unsigned char Echo_5[80];
int Idx_5=0;
void DoCom_5(void) /* COM5=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_5()){
    if(WaitCom1Echo_5){
      data=ReadCom_5();
      LastHitTimeTick= * timetick;
      Cmd_5[Idx_5++]=data;
      if(data=='\r'){
	if(ModeCrLf_5==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_5[Idx_5-1]=0;
	  SendCmdToCom485Str(Cmd_5,Idx_5-1);        /* send command-string to COM2 */
	  Idx_5=0;
	  reply=1;
	}
	else if(ModeCrLf_5==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_5[Idx_5-2]=='\n'){
		  Cmd_5[Idx_5-2]=0;
		  SendCmdToCom485Str(Cmd_5,Idx_5-2);/* send command-string to COM2 */
		  Idx_5=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_5==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_5[Idx_5-1]=0;
	       SendCmdToCom485Str(Cmd_5,Idx_5-1);   /* send command-string to COM2 */
	       Idx_5=0;
	       reply=1;
	    }
	    else if(ModeCrLf_5==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_5[Idx_5-2]=='\r'){
		   Cmd_5[Idx_5-2]=0;
		   SendCmdToCom485Str(Cmd_5,Idx_5-2);/* send command-string to COM2 */
		   Idx_5=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_5(ReadCom_5());
    }
  }
  if( (ModeCrLf_5==4) &&(Idx_5!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_5 )
     {
	   SendCmdToCom485Str(Cmd_5,Idx_5);
	   Idx_5=0;
	   reply=1;
     }

  }
  if(WaitCom1Echo_5&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IdleTime_5+IDLETOOLONG) )
     {
	WaitCom1Echo_5=0;
	reply=0;
     }
  }
}

///////////////////////////////////////////////////////////////////
//COM6
unsigned char RingBuf_6[1024];
int InIdx_6=0,OutIdx_6=0;
int WaitCom1Echo_6=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_6(void)
{
  InIdx_6=OutIdx_6=0;
}

void AddDataToRingBuf_6(unsigned char data)
{
   if(((InIdx_6+1)&1023) != OutIdx_6){
       RingBuf_6[InIdx_6++]=data;
       if(InIdx_6>=1024) InIdx_6=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_6(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_6 != OutIdx_6)
   {
      *buf=(data=RingBuf_6[OutIdx_6]);
      buf++;
      i++;
      OutIdx_6++; if(OutIdx_6>=1024) OutIdx_6=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_6==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_6==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_6==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_6==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM6 --> bypass to COM2 -- */

unsigned char Cmd_6[256];
//unsigned char Echo_6[80];
int Idx_6=0;
void DoCom_6(void) /* COM6=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_6()){
    if(WaitCom1Echo_6){
      data=ReadCom_6();
      LastHitTimeTick= * timetick;
      Cmd_6[Idx_6++]=data;
      if(data=='\r'){
	if(ModeCrLf_5==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_6[Idx_6-1]=0;
	  SendCmdToCom485Str(Cmd_6,Idx_6-1);        /* send command-string to COM2 */
	  Idx_6=0;
	  reply=1;
	}
	else if(ModeCrLf_6==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_6[Idx_6-2]=='\n'){
		  Cmd_6[Idx_6-2]=0;
		  SendCmdToCom485Str(Cmd_6,Idx_6-2);/* send command-string to COM2 */
		  Idx_6=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_6==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_6[Idx_6-1]=0;
	       SendCmdToCom485Str(Cmd_6,Idx_6-1);   /* send command-string to COM2 */
	       Idx_6=0;
	       reply=1;
	    }
	    else if(ModeCrLf_6==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_6[Idx_6-2]=='\r'){
		   Cmd_6[Idx_6-2]=0;
		   SendCmdToCom485Str(Cmd_6,Idx_6-2);/* send command-string to COM2 */
		   Idx_6=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_6(ReadCom_6());
    }
  }
  if( (ModeCrLf_6==4) &&(Idx_6!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_6)
     {
	   SendCmdToCom485Str(Cmd_6,Idx_6);
	   Idx_6=0;
	   reply=1;
     }

  }
  if(WaitCom1Echo_6&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IdleTime_6+IDLETOOLONG) )
     {
	WaitCom1Echo_6=0;
	reply=0;
     }
  }
}


///////////////////////////////////////////////////////////////////
//COM7
unsigned char RingBuf_7[1024];
int InIdx_7=0,OutIdx_7=0;
int WaitCom1Echo_7=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_7(void)
{
  InIdx_7=OutIdx_7=0;
}

void AddDataToRingBuf_7(unsigned char data)
{
   if(((InIdx_7+1)&1023) != OutIdx_7){
       RingBuf_7[InIdx_7++]=data;
       if(InIdx_7>=1024) InIdx_7=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_7(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_7 != OutIdx_7)
   {
      *buf=(data=RingBuf_7[OutIdx_7]);
      buf++;
      i++;
      OutIdx_7++; if(OutIdx_7>=1024) OutIdx_7=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_7==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_7==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_7==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_7==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM7 --> bypass to COM2 -- */

unsigned char Cmd_7[256];
//unsigned char Echo_7[80];
int Idx_7=0;
void DoCom_7(void) /* COM7=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_7()){
    if(WaitCom1Echo_7){
      data=ReadCom_7();
      LastHitTimeTick= * timetick;
      Cmd_7[Idx_7++]=data;
      if(data=='\r'){
	if(ModeCrLf_7==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_7[Idx_7-1]=0;
	  SendCmdToCom485Str(Cmd_7,Idx_7-1);        /* send command-string to COM2 */
	  Idx_7=0;
	  reply=1;
	}
	else if(ModeCrLf_7==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_7[Idx_7-2]=='\n'){
		  Cmd_7[Idx_7-2]=0;
		  SendCmdToCom485Str(Cmd_7,Idx_7-2);/* send command-string to COM2 */
		  Idx_7=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_7==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_7[Idx_7-1]=0;
	       SendCmdToCom485Str(Cmd_7,Idx_7-1);   /* send command-string to COM2 */
	       Idx_7=0;
	       reply=1;
	    }
	    else if(ModeCrLf_7==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_7[Idx_7-2]=='\r'){
		   Cmd_7[Idx_7-2]=0;
		   SendCmdToCom485Str(Cmd_7,Idx_7-2);/* send command-string to COM2 */
		   Idx_7=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_7(ReadCom_7());
    }
  }
  if( (ModeCrLf_7==4) &&(Idx_7!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_7)
     {
	   SendCmdToCom485Str(Cmd_7,Idx_7);
	   Idx_7=0;
	   reply=1;
     }

  }
  if(WaitCom1Echo_7&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IdleTime_7+IDLETOOLONG) )
     {
	WaitCom1Echo_7=0;
	reply=0;
     }

  }
}


///////////////////////////////////////////////////////////////////
//COM8
unsigned char RingBuf_8[1024];
int InIdx_8=0,OutIdx_8=0;
int WaitCom1Echo_8=0;
/* ------------------------------------------------------------- */
void ResetRingBuf_8(void)
{
  InIdx_8=OutIdx_8=0;
}

void AddDataToRingBuf_8(unsigned char data)
{
   if(((InIdx_8+1)&1023) != OutIdx_8){
       RingBuf_8[InIdx_8++]=data;
       if(InIdx_8>=1024) InIdx_8=0;
   }
   else {
      /* buffer is full !!! */
      ;
   }
}

int GetDataInRingBuf_8(unsigned char *buf)
{

/////////////
   unsigned char data;
   int i=0;
   while(InIdx_8 != OutIdx_8)
   {
      *buf=(data=RingBuf_8[OutIdx_8]);
      buf++;
      i++;
      OutIdx_8++; if(OutIdx_8>=1024) OutIdx_8=0;/* move to next*/
      if(data=='\r'){
	 if(ModeCrLf_8==0){		      /* CrLfMode=0=CR --> match     */
	     *(buf-1)=0;
	     return i;
	 }
	 else if(ModeCrLf_8==3){	      /* CrLfMode=3=LF+CR --> match  */
		  if(*(buf-2)=='\n'){
		     *(buf-2)=0;
		     return i; /* send command-string to COM2 */
		  }
	 }
       }
       else if(data=='\n'){
	       if(ModeCrLf_8==2){	      /* CrLfMode=2=LF --> match     */
		  *(buf-1)=0;
		  return i;    /* send command-string to COM2 */
	       }
	       else if(ModeCrLf_8==1){       /* CrLfMode=1=CR+LF --> match  */
		    if(*(buf-2)=='\r'){
		       *(buf-2)=0;
		       return i;/* send command-string to COM2 */
		    }
	       }
       }
   }
   *buf=0;
/////////////
   return i;
}


/* --- if receive a commmad-string from COM8 --> bypass to COM2 -- */

unsigned char Cmd_8[256];
//unsigned char Echo_8[80];
int Idx_8=0;
void DoCom_8(void) /* COM8=RS-232 */
{ int data;
  static unsigned long LastHitTimeTick;
  static int reply=0;
  while(IsCom_8()){
    if(WaitCom1Echo_8){
      data=ReadCom_8();
      LastHitTimeTick= * timetick;
      Cmd_8[Idx_8++]=data;
      if(data=='\r'){
	if(ModeCrLf_8==0){ 	       /* CrLfMode=0=CR --> match     */
	  Cmd_8[Idx_8-1]=0;
	  SendCmdToCom485Str(Cmd_8,Idx_8-1);        /* send command-string to COM2 */
	  Idx_8=0;
	  reply=1;
	}
	else if(ModeCrLf_8==3){	       /* CrLfMode=3=LF+CR --> match  */
	       if(Cmd_8[Idx_8-2]=='\n'){
		  Cmd_8[Idx_8-2]=0;
		  SendCmdToCom485Str(Cmd_8,Idx_8-2);/* send command-string to COM2 */
		  Idx_8=0;
		  reply=1;
	       }
	}
      }
      else if(data=='\n'){
	    if(ModeCrLf_8==2){	       /* CrLfMode=2=LF --> match     */
	       Cmd_8[Idx_8-1]=0;
	       SendCmdToCom485Str(Cmd_8,Idx_8-1);   /* send command-string to COM2 */
	       Idx_8=0;
	       reply=1;
	    }
	    else if(ModeCrLf_8==1){	/* CrLfMode=1=CR+LF --> match  */
		 if(Cmd_8[Idx_8-2]=='\r'){
		   Cmd_8[Idx_8-2]=0;
		   SendCmdToCom485Str(Cmd_8,Idx_8-2);/* send command-string to COM2 */
		   Idx_8=0;
		   reply=1;
		 }
	    }
      }
    }
    else {
       AddDataToRingBuf_8(ReadCom_8());
    }
  }
  if( (ModeCrLf_8==4) &&(Idx_8!=0) )
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime_8)
     {
	   SendCmdToCom485Str(Cmd_8,Idx_8);
	   Idx_8=0;
	   reply=1;
     }
  }
  if(WaitCom1Echo_8&&(reply==1))
  {
     if( ((*timetick)-LastHitTimeTick)>(IdleTime_8+IDLETOOLONG) )
     {
	WaitCom1Echo_8=0;
	reply=0;
     }
  }
}

/* ------------------------------------------------------------- */

int addr7527;
unsigned char Echo485[1024];
void EchoErrorCmd(void) 		/* echo ?AA to host */
{
  sprintf(Echo485,"?%02X",addr7527);
  SendCmdToCom485Str(Echo485,strlen(Echo485));
}

void EchoOkCmd(void)			/* echo !AA to host */
{
  sprintf(Echo485,"!%02X",addr7527);
  SendCmdToCom485Str(Echo485,strlen(Echo485));
}

int EchoType=0;
/*
EchoType:
0: NoEcho
1: call EchoErrorCmd();
2: call EchoOkCmd();
3: Echo the string in Echo2
*/
enum ECHO_TYPE { NO_ECHO=0,ERROR_ECHO,OK_ECHO,ECHO2_ECHO };
unsigned char Cmd485[256];
int Idx485=0;
int Idx485cmd=0;
int InitPin;  /* 1: Init* connect to GND */

void Quit(void)
{
   sprintf(Echo485,"!%02XQuit7527",Address);
   EchoType=ECHO2_ECHO;
   quit=1;
}

/* --- set ID-string for COM1/3   ---------------------------------- */

void SetID(void)
{ int addr;
  int start,end;
  int bank;
  char *dataptr=Cmd485+4;

  EnableEEP();
  if(addr7527==Address){		/* ID-string for COM1(RS232) */
     start=150;
     end=200;
     bank=7;
  }
  else if(addr7527==Address+1){	/* ID-string for COM3(RS232) */
     start=200;
     end=250;
     bank=7;
  }
///
  else if(addr7527==Address+2){	/* ID-string for COM4(RS232) */
     start=0;
     end=50;
     bank=6;
  }
  else if(addr7527==Address+3){	/* ID-string for COM5(RS232) */
     start=50;
     end=100;
     bank=6;
  }
  else if(addr7527==Address+4){	/* ID-string for COM6(RS232) */
     start=100;
     end=150;
     bank=6;
  }
  else if(addr7527==Address+5){	/* ID-string for COM7(RS232) */
     start=150;
     end=200;
     bank=6;
  }
  else if(addr7527==Address+6){	/* ID-string for COM8(RS232) */
     start=200;
     end=250;
     bank=6;
  }

  for(addr=start; addr<end && *dataptr; addr++){
      WriteEEP(bank,addr,*dataptr++);
  }
  WriteEEP(bank,addr,0);
  ProtectEEP();
  EchoType=OK_ECHO;
}

void ReadID(void) /* read the ID-string of COM1/3		     */
{ int data;
  int addr;
  int end;
  int bank;
  char *ptr=Echo485+3;

   if(addr7527==Address){	     /* ID-string of COM1(RS232)     */
      addr=150;
      end=200;
      bank=7;
   }
   else if(addr7527==Address+1){  /* ID-string of COM3(RS232)     */
      addr=200;
      end=250;
      bank=7;
   }

//////
  else if(addr7527==Address+2){	/* ID-string for COM4(RS232) */
     addr=0;
     end=50;
     bank=6;
  }

  else if(addr7527==Address+3){	/* ID-string for COM5(RS232) */
     addr=50;
     end=100;
     bank=6;
  }
  else if(addr7527==Address+4){	/* ID-string for COM6(RS232) */
     addr=100;
     end=150;
     bank=6;
  }
  else if(addr7527==Address+5){	/* ID-string for COM7(RS232) */
     addr=150;
     end=200;
     bank=6;
  }
  else if(addr7527==Address+6){	/* ID-string for COM8(RS232) */
     addr=200;
     end=250;
     bank=6;
  }

   sprintf(Echo485,"!%02X",addr7527);/*!AA                            */
   do { 			     /*!AA(ID-string)		      */
     *ptr++=data=ReadEEP(bank,addr++);
   } while(data && addr<end);
   *ptr=0;
  EchoType=ECHO2_ECHO;
}

/* --- Read/set module address (AA) --------------------------------- */

void ReadAddress(void)	/* read AA stored in EEPROM		      */
{
  sprintf(Echo485,"!%02X",ReadEEP(7,50));
  EchoType=ECHO2_ECHO;
}

void SetAddress(void)
{
  Address=(ascii_to_hex(Cmd485[4])<<4)+ascii_to_hex(Cmd485[5]);
  EnableEEP();
  WriteEEP(7,50,Address);
  ProtectEEP();
  Address=Address;
					  /* AA show in (LED1,LED2)   */
  Show5DigitLed(1,Address>>4);	  /* high nibble of AA	      */
  Show5DigitLedWithDot(2,Address&0x0F);/* low  nibble of AA	      */

  EchoType=OK_ECHO;
}

/* --- read the baud-rate for COM1/2/3	 ---------------------------- */

void ReadComBaudrate(void)	/* read baudrate stored in EEPROM     */
{
   long data;
   if(addr7527==Address){			      /* AA	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,6)];/* COM1(RS232)  */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+1){		      /* AA+1	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,10)];/* COM3(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+2){		      /* AA+2	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,51)];/* COM4(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+3){		      /* AA+3	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,55)];/* COM5(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+4){		      /* AA+4	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,62)];/* COM6(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+5){		      /* AA+5	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,66)];/* COM7(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
   else if(addr7527==Address+6){		      /* AA+6	      */
      if(Cmd485[4]=='0') data=BAUD[ReadEEP(7,2)];     /* COM2(RS485)  */
      else if(Cmd485[4]=='1') data=BAUD[ReadEEP(7,70)];/* COM8(RS232) */
      else return;
      sprintf(Echo485,"!%02X%ld",addr7527,data);
   }
  EchoType=ECHO2_ECHO;
}

void SetComBaudrate(void)	/* set the baud-rate of COM-1/2/3/4   */
{
  long data;
//  long baud;
  int i;

/*  if(!InitPin){
     EchoType=ERROR_ECHO;
     return;
  } */ /* mark for need not INIT* connect to GND */
  Cmd485[Idx485cmd]=0;// make sure 0x00 is  the end for string-converting
/*
  if(DEBUG_7527)
  {
     dPrint(" %s",Cmd485);
  }
*/
  data=atol(Cmd485+5);
/*
  if(DEBUG_7527)
  {
     dPrint(" %ld",data);

  }
*/
  for(i=1;i<=10;i++)
  {
//     baud=BAUD[i];
/*
     if(DEBUG_7527)
     {
	dPrint(" %ld",data);
	dPrint("BAUD[%d]=%ld",i,BAUD[i]);
     }
*/
     if(data==BAUD[i]) goto SetBaud;
  }
  return;
SetBaud:
  if(addr7527==Address){			/* AA		      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM1(RS232)        */
	if(DEBUG_7527)
	   dPrint(" i=%d",i);
	Baud232=i;
	EnableEEP();
	WriteEEP(7,6,Baud232);
	ProtectEEP();
	if(DEBUG_7527)
	   dPrint(" Baud rate change complete");
	if(!DEBUG_7527)
	{
	   RestoreCom232();			/* change immediately */
	   InstallCom232(BAUD[Baud232],Data232,Parity232);
	}
	Show5DigitLedWithDot(4,Baud232);	/* baud rate of COM2  */
     }
     else return;
  }
  else if(addr7527==Address+1){		/* AA+1 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM3(RS232)        */
	Baud_3=i;
	EnableEEP();
	WriteEEP(7,10,Baud_3);
	ProtectEEP();
	RestoreCom_36();			/* change immediately */
	InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	SetBaudrate_3(BAUD[Baud_3]);
	SetDataFormat_3(Data_3,Parity_3,Stop_3);
     }
     else return;
  }
  else if(addr7527==Address+2){		/* AA+2 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM4(RS232)        */
	Baud_4=i;
	EnableEEP();
	WriteEEP(7,51,Baud_4);
	ProtectEEP();
	RestoreCom_47();			/* change immediately */
	InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	SetBaudrate_4(BAUD[Baud_4]);
	SetDataFormat_4(Data_4,Parity_4,Stop_4);
     }
     else return;
  }
  else if(addr7527==Address+3){		/* AA+3 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM5(RS232)        */
	Baud_5=i;
	EnableEEP();
	WriteEEP(7,55,Baud_5);
	ProtectEEP();
	RestoreCom_58();			/* change immediately */
	InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	SetBaudrate_5(BAUD[Baud_5]);
	SetDataFormat_5(Data_5,Parity_5,Stop_5);

     }
     else return;
  }
  else if(addr7527==Address+4){		/* AA+4 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM6(RS232)        */
	Baud_6=i;
	EnableEEP();
	WriteEEP(7,62,Baud_6);
//	printCom1(" Baud_6=%d",Baud_6);
	ProtectEEP();
	RestoreCom_36();			/* change immediately */
	InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	SetBaudrate_3(BAUD[Baud_3]);
	SetDataFormat_3(Data_3,Parity_3,Stop_3);
     }
     else return;
  }
  else if(addr7527==Address+5){		/* AA+5 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM7(RS232)        */
	Baud_7=i;
	EnableEEP();
	WriteEEP(7,66,Baud_7);
	ProtectEEP();
	RestoreCom_47();			/* change immediately */
	InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	SetBaudrate_4(BAUD[Baud_4]);
	SetDataFormat_4(Data_4,Parity_4,Stop_4);
     }
     else return;
  }
  else if(addr7527==Address+6){		/* AA+6 	      */
     if(Cmd485[4]=='0'){                        /* COM2(RS485)        */
	Baud485=i;
	EnableEEP();
	WriteEEP(7,2,Baud485);
	ProtectEEP();
	WaitToDo=_ChangeBaudrate;		/* change after echo  */
	Show5DigitLedWithDot(3,Baud485);	/* baud rate of COM2  */
     }
     else if(Cmd485[4]=='1'){                   /* COM8(RS232)        */
	Baud_8=i;
	EnableEEP();
	WriteEEP(7,70,Baud_8);
	ProtectEEP();
	RestoreCom_58();			/* change immediately */
	InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	SetBaudrate_5(BAUD[Baud_5]);
	SetDataFormat_5(Data_5,Parity_5,Stop_5);
     }
     else return;
  }
  IdleTime485=1+(int)(11000.0/(float)BAUD[Baud485]);
  IdleTime232=1+(int)(11000.0/(float)BAUD[Baud232]);
  IdleTime_3=1+(int)(11000.0/(float)BAUD[Baud_3]);
  IdleTime_4=1+(int)(11000.0/(float)BAUD[Baud_4]);
  IdleTime_5=1+(int)(11000.0/(float)BAUD[Baud_5]);
  IdleTime_6=1+(int)(11000.0/(float)BAUD[Baud_6]);
  IdleTime_7=1+(int)(11000.0/(float)BAUD[Baud_7]);
  IdleTime_8=1+(int)(11000.0/(float)BAUD[Baud_8]);

  EchoType=OK_ECHO;
}

/* ------------------------------------------------------------------ */

void SetDelimiter(void)
{
  EnableEEP();
  if(addr7527==Address){			/* AA		      */
     WriteEEP(7,31,(Delimiter=Cmd485[4]));	/* COM1(RS232)	      */
  }
  else if(addr7527==Address+1){		/* AA+1 	      */
     WriteEEP(7,32,(Delimiter_3=Cmd485[4]));	/* COM3(RS232)	      */
  }
  else if(addr7527==Address+2){		/* AA+2 	      */
     WriteEEP(7,33,(Delimiter_4=Cmd485[4]));	/* COM4(RS232)	      */
  }
  else if(addr7527==Address+3){		/* AA+3 	      */
     WriteEEP(7,34,(Delimiter_5=Cmd485[4]));	/* COM5(RS232)	      */
  }
  else if(addr7527==Address+4){		/* AA+4 	      */
     WriteEEP(7,35,(Delimiter_6=Cmd485[4]));	/* COM6(RS232)	      */
  }
  else if(addr7527==Address+5){		/* AA+5 	      */
     WriteEEP(7,36,(Delimiter_7=Cmd485[4]));	/* COM7(RS232)	      */
  }
  else if(addr7527==Address+6){		/* AA+6 	      */
     WriteEEP(7,37,(Delimiter_8=Cmd485[4]));	/* COM8(RS232)	      */
  }
  ProtectEEP();
  EchoType=OK_ECHO;
}

void ReadDelimiter(void) /* read delimiter stored in EEPROM	      */
{
  int t;
  if(addr7527==Address){
     t=ReadEEP(7,31);
  }
  else if(addr7527==(Address+1)){
     t=ReadEEP(7,32);
  }
  else if(addr7527==(Address+2)){
     t=ReadEEP(7,33);
  }
  else if(addr7527==(Address+3)){
     t=ReadEEP(7,34);
  }
  else if(addr7527==(Address+4)){
     t=ReadEEP(7,35);
  }
  else if(addr7527==(Address+5)){
     t=ReadEEP(7,36);
  }
  else if(addr7527==(Address+6)){
     t=ReadEEP(7,37);
  }
  sprintf(Echo485,"!%02X%c",addr7527,t);
  EchoType=ECHO2_ECHO;
}

/* --------------------------------------------------------------------- */

void ReadDataBit(void)	/* read data-bit stored in EEPROM		 */
{ int data;

  if(Cmd485[4]=='0') data=ReadEEP(7,4);                     /* COM2      */
  else if(Cmd485[4]=='1'){
	   if(addr7527==Address) data= ReadEEP(7,8);     /* AA=COM1	 */
	   else if(addr7527==Address+1) data=ReadEEP(7,12);/*AA+1=COM3*/
	   else if(addr7527==Address+2) data=ReadEEP(7,53);/*AA+2=COM4*/
	   else if(addr7527==Address+3) data=ReadEEP(7,57);/*AA+3=COM5*/
	   else if(addr7527==Address+4) data=ReadEEP(7,64);/*AA+4=COM6*/
	   else if(addr7527==Address+5) data=ReadEEP(7,68);/*AA+5=COM7*/
	   else if(addr7527==Address+6) data=ReadEEP(7,72);/*AA+6=COM8*/
  }
  else return;
  sprintf(Echo485,"!%02X%d",addr7527,data);
  EchoType=ECHO2_ECHO;
}

void SetDataBit(void)
{ int data;

/*  if(!InitPin){
     EchoType=ERROR_ECHO;
     return;
  } */	/* mark for need not INIT* connect to GND	*/
  if(Cmd485[5]=='7') data=7;      /* only 7/8 are valid */
  else if(Cmd485[5]=='8') data=8; /* only 7/8 are valid */
  else return;

  if(Cmd485[4]=='0'){                           /* AA/AA+1/AA+2=COM2  */
     Data485=data;
     EnableEEP();
     WriteEEP(7,4,Data485);
     ProtectEEP();
     WaitToDo=_ChangeDataBit;			/* change after echo  */
  }
  else if(Cmd485[4]=='1'){
	  EnableEEP();
	  if(addr7527==Address){		/* AA=COM1(RS232)     */
	     WriteEEP(7,8,(Data232=data));
	     RestoreCom232();			/* change immediately */
	     InstallCom232(BAUD[Baud232],Data232,Parity232);
	  }
	  else if(addr7527==Address+1){	/* AA+1=COM3(RS232)   */
	     WriteEEP(7,12,(Data_3=data));
	     RestoreCom_36(); 		/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+2){	/* AA+2=COM4(RS232)   */
	     WriteEEP(7,53,(Data_4=data));
	     RestoreCom_47(); 		/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+3){	/* AA+3=COM5(RS232)   */
	     WriteEEP(7,57,(Data_5=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
	  else if(addr7527==Address+4){	/* AA+4=COM6(RS232)   */
	     WriteEEP(7,64,(Data_6=data));
	     RestoreCom_36();			/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+5){	/* AA+5=COM7(RS232)   */
	     WriteEEP(7,68,(Data_7=data));
	     RestoreCom_47();			/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+6){	/* AA+6=COM8(RS232)   */
	     WriteEEP(7,72,(Data_8=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
	  ProtectEEP();
  }
  else return;
  EchoType=OK_ECHO;
}

/* --- read the end-char mode ------------------------------------------ */

void ReadCRLF(void) /* read end-char-mode stored in EEPROM		 */
{
   int data;
   if(DEBUG_7527) dPrint("addr7527=%d  Address=%d",addr7527,Address);
   if(Cmd485[4]=='0') data=ReadEEP(7,40);                   /* COM2      */
   else if(Cmd485[4]=='1'){
      if(addr7527==Address) data=ReadEEP(7,41);	    /* AA=COM1	 */
      else if(addr7527==(Address+1)) data=ReadEEP(7,42);   /* AA+1=COM3 */
      else if(addr7527==(Address+2)) data=ReadEEP(7,43);   /* AA+2=COM4 */
      else if(addr7527==(Address+3)) data=ReadEEP(7,44);   /* AA+3=COM5 */
      else if(addr7527==(Address+4)) data=ReadEEP(7,45);   /* AA+4=COM6 */
      else if(addr7527==(Address+5)) data=ReadEEP(7,46);   /* AA+5=COM7 */
      else if(addr7527==(Address+6)) data=ReadEEP(7,47);   /* AA+6=COM8 */
   }
   else return;
  sprintf(Echo485,"!%02X%d",addr7527,data);
  EchoType=ECHO2_ECHO;
}

void SetCRLF(void)
{
  int data;

  if(Cmd485[5]>='0' && Cmd485[5]<='4') data=Cmd485[5]-'0';
  else return;
  if(Cmd485[4]=='0'){                           /* AA/AA+1/AA+2=COM2  */
     ModeCrLf485=data;
     ClearCom2();
     EnableEEP();
     WriteEEP(7,40,ModeCrLf485);
     ProtectEEP();
  }
  else if(Cmd485[4]=='1'){
     EnableEEP();
     if(addr7527==Address){			/* AA=COM1(RS232)     */
	ClearCom2();
	WriteEEP(7, 41,(ModeCrLf232=data));}
     else if(addr7527==Address+1){		/* AA+1=COM3(RS232)   */
	WriteEEP(7, 42,(ModeCrLf_3=data));
	ClearCom_3();}
     else if(addr7527==Address+2){		/* AA+2=COM4(RS232)   */
	WriteEEP(7, 43,(ModeCrLf_4=data));
	ClearCom_4();}
     else if(addr7527==Address+3){		/* AA+3=COM5(RS232)   */
	WriteEEP(7, 44,(ModeCrLf_5=data));
	ClearCom_5();}
     else if(addr7527==Address+4){		/* AA+4=COM6(RS232)   */
	WriteEEP(7, 45,(ModeCrLf_6=data));
	ClearCom_6();}
     else if(addr7527==Address+5){		/* AA+5=COM7(RS232)   */
	WriteEEP(7, 46,(ModeCrLf_7=data));
	ClearCom_7();}
     else if(addr7527==Address+6){		/* AA+6=COM8(RS232)   */
	WriteEEP(7, 47,(ModeCrLf_8=data));
	ClearCom_8();}
     ProtectEEP();
  }
  else return;
  EchoType=OK_ECHO;
}

/* --- read the parity-bit --------------------------------------------- */

void ReadParity(void) /* read parity-bit stored in EEPROM		 */
{ int data;

  if(Cmd485[4]=='0') data=ReadEEP(7,3);                    /* COM2       */
  else if(Cmd485[4]=='1'){
	 if(addr7527==Address)  data=ReadEEP(7,7);	   /* AA=COM1	 */
	 else if(addr7527==Address+1) data=ReadEEP(7,11);/* AA+1=COM3 */
	 else if(addr7527==Address+2) data=ReadEEP(7,52);/* AA+2=COM4 */
	 else if(addr7527==Address+3) data=ReadEEP(7,56);/* AA+3=COM5 */
	 else if(addr7527==Address+4) data=ReadEEP(7,63);/* AA+4=COM6 */
	 else if(addr7527==Address+5) data=ReadEEP(7,67);/* AA+5=COM7 */
	 else if(addr7527==Address+6) data=ReadEEP(7,71);/* AA+6=COM8 */
  }
  else return;
  sprintf(Echo485,"!%02X%d",addr7527,data);
  EchoType=ECHO2_ECHO;
}

void SetParity(void)
{
 int data;

 if(Cmd485[5]=='0') data=0;     /* valid 0/1/2    */
 else if(Cmd485[5]=='1') data=1;/* 0=None         */
 else if(Cmd485[5]=='2') data=2;/* 1=Even         */
 else return;			/* 2=Odd	  */
 if(Cmd485[4]=='0'){                            /* COM2(RS485)        */
    Parity485=data;
    EnableEEP();
    WriteEEP(7,3,Parity485);
    ProtectEEP();
    WaitToDo=_ChangeParity;			/* change later       */
 }
 else if(Cmd485[4]=='1'){                       /* AA=COM1(RS232)     */
    EnableEEP();
    if(addr7527==Address){
       WriteEEP(7,7,(Parity232=data));
       RestoreCom232(); 			/* change immediately */
       InstallCom232(BAUD[Baud232],Data232,Parity232);
    }
///////////////////////
	  else if(addr7527==Address+1){	/* AA+1=COM3(RS232)   */
	     WriteEEP(7,11,(Parity_3=data));
	     RestoreCom_36(); 		/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+2){	/* AA+2=COM4(RS232)   */
	     WriteEEP(7,52,(Parity_4=data));
	     RestoreCom_47(); 		/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+3){	/* AA+3=COM5(RS232)   */
	     WriteEEP(7,56,(Parity_5=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
	  else if(addr7527==Address+4){	/* AA+4=COM6(RS232)   */
	     WriteEEP(7,63,(Parity_6=data));
	     RestoreCom_36();			/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+5){	/* AA+5=COM7(RS232)   */
	     WriteEEP(7,67,(Parity_7=data));
	     RestoreCom_47();			/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+6){	/* AA+6=COM8(RS232)   */
	     WriteEEP(7,71,(Parity_8=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
    ProtectEEP();
 }
 else return;
 EchoType=OK_ECHO;
}

/* --- read the stop-bit -------------------------------------------- */

void ReadStopBit(void) /* read stop-bit stored in EEPROM	      */
{
   int data;					/* can't change    ?? */
   if(Cmd485[4]=='0') data=ReadEEP(7,5);        /* COM2(RS485)        */
   else if(Cmd485[4]=='1') {
	   if(addr7527==Address)		/* AA=COM1(RS232)     */
	      data=ReadEEP(7,9);
	   else if(addr7527==Address+1)
	      data=ReadEEP(7,13);		/* AA+1=COM3(RS232)   */
	   else if(addr7527==Address+2)
	      data=ReadEEP(7,54);		/* AA+2=COM4(RS232)   */
	   else if(addr7527==Address+3)
	      data=ReadEEP(7,58);		/* AA+3=COM5(RS232)   */
	   else if(addr7527==Address+4)
	      data=ReadEEP(7,65);		/* AA+4=COM6(RS232)   */
	   else if(addr7527==Address+5)
	      data=ReadEEP(7,69);		/* AA+5=COM7(RS232)   */
	   else if(addr7527==Address+6)
	      data=ReadEEP(7,73);		/* AA+6=COM8(RS232)   */
   }
   else return;
  sprintf(Echo485,"!%02X%d",addr7527,data);
  EchoType=ECHO2_ECHO;
}

void SetStopBit(void)/*$AAOXN*/
		     /*012345*/
{
 int data;

 if(Cmd485[5]=='1') data=1;      /* valid 1/2     */
 else if(Cmd485[5]=='2') data=2; /* valid 1/2     */
 else return;

 if(Cmd485[4]=='0'){             /* can't change  */
    return;
 }

 else if(Cmd485[4]=='1')         /* 232 port */
 {
    EnableEEP();
    if(addr7527==Address)/* 1st 232 */
    {
       return;
    }

///
	  else if(addr7527==Address+1){	/* AA+1=COM3(RS232)   */
	     WriteEEP(7,13,(Stop_3=data));
	     RestoreCom_36(); 		/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+2){	/* AA+2=COM4(RS232)   */
	     WriteEEP(7,54,(Stop_4=data));
	     RestoreCom_47(); 		/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+3){	/* AA+3=COM5(RS232)   */
	     WriteEEP(7,58,(Stop_5=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
	  else if(addr7527==Address+4){	/* AA+4=COM6(RS232)   */
	     WriteEEP(7,65,(Stop_6=data));
	     RestoreCom_36();			/* change immediately */
	     InstallCom_36(BAUD[Baud_6],Data_6,Parity_6,Stop_6);
	     SetBaudrate_3(BAUD[Baud_3]);
	     SetDataFormat_3(Data_3,Parity_3,Stop_3);
	  }
	  else if(addr7527==Address+5){	/* AA+5=COM7(RS232)   */
	     WriteEEP(7,69,(Stop_7=data));
	     RestoreCom_47();			/* change immediately */
	     InstallCom_47(BAUD[Baud_7],Data_7,Parity_7,Stop_7);
	     SetBaudrate_4(BAUD[Baud_4]);
	     SetDataFormat_4(Data_4,Parity_4,Stop_4);
	  }
	  else if(addr7527==Address+6){	/* AA+6=COM8(RS232)   */
	     WriteEEP(7,73,(Stop_8=data));
	     RestoreCom_58();			/* change immediately */
	     InstallCom_58(BAUD[Baud_8],Data_8,Parity_8,Stop_8);
	     SetBaudrate_5(BAUD[Baud_5]);
	     SetDataFormat_5(Data_5,Parity_5,Stop_5);
	  }
    ProtectEEP();
 }
 else return;
 EchoType=OK_ECHO;
}

/* --- read the check-sum enable/disable ----------------------- */

void ReadCheckSumMode(void) /* read checksum status in EEPROM	 */
{
  int data=ReadEEP(7,20);

  sprintf(Echo485,"!%02X%d",addr7527,data);
  EchoType=ECHO2_ECHO;
}

void SetCheckSumMode(void)
{ int data;

  if(Cmd485[4]=='1') data=1;            /* enable checksum       */
  else if(Cmd485[4]=='0') data=0;       /* disable checksum      */
  else {
     return;
  }

  EnableEEP();
  NewCheckSum=data;
  WaitToDo=_ChangeCheckSum;		/* change after echo	 */
  WriteEEP(7,20,data);
  ProtectEEP();
  EchoType=OK_ECHO;
}

/* --- read the configuration of COM2(RS485) stored in EEPROM ---------- */

void ReadConfiguration(void)
{ int BB,DD,PP,KK;

  BB=ReadEEP(7,2); /* baud rate       */
  DD=ReadEEP(7,4); /* data-bit	      */
  PP=ReadEEP(7,3); /* parity-bit      */
  KK=ReadEEP(7,20);/* checksum status */
  sprintf(Echo485,"!%02X40%x%x%x%x",addr7527,BB,DD,PP,KK);
  EchoType=ECHO2_ECHO;
}

/* --- Read the CTS of COM1 & COM3 ----------------------------- */
/* return !AA0: CTS inactive now				 */
/* return !AA1: CTS active LOW now				 */

void ReadCts(void) /* valid address: AA & AA+1			 */
{
  int data;
  if  (addr7527==Address)	   /* AA=COM1			 */
      sprintf(Echo485,"!%02X%d",addr7527,GetCtsStatus1()?1:0);
  else
      sprintf(Echo485,"?%02X",addr7527);
  EchoType=ECHO2_ECHO;
}

/* --- Set the RTS of COM1 & COM3 ------------------------------ */
/* $AAW0: RTS inactive						 */
/* $AAW1: RTS active LOW					 */

void SetRts(void) /* valid address: AA 			 */
{
  int data;
  if  (addr7527==Address)	   /* AA=COM1			 */
      {
      if(Cmd485[4]=='0')           /* Rts of COM1 inactive       */
	{
	SetRtsInactive1();
	sprintf(Echo485,"!%02X",addr7527);
	}
      else if(Cmd485[4]=='1')      /* Rts of COM1 active LOW     */
	{
	SetRtsActive1();
	sprintf(Echo485,"!%02X",addr7527);
	}
      else
      sprintf(Echo485,"?%02X",addr7527);
      }
  else
      sprintf(Echo485,"?%02X",addr7527);
  EchoType=ECHO2_ECHO;
}

/* --- Read On Board DI ---------------------------------------- */
/* $AAY1: read DI1						 */

void ReadOnBoardDi(void) /* valid address: AA			 */
{
  int data;
  if  (addr7527==Address)	   /* AA			 */
      {
      if(Cmd485[4]=='1')           /* Read DI1=INIT* pin         */
	{			   /* DI1=!ReadInitPin()	 */
	sprintf(Echo485,"!%02X%d",addr7527,GetDi1()?1:0);
	}
      else
      sprintf(Echo485,"?%02X",addr7527);
      }
  else
      sprintf(Echo485,"?%02X",addr7527);
  EchoType=ECHO2_ECHO;
}

/* --- Write On Board DO --------------------------------------- */
/* $AAZ10: Turn DO1 OFF, $AAZ11:Turn DO1 ON			 */
char onbdo=0;
void WriteOnBoardDo(void) /* valid address: AA			 */
{
  int data;
  if  (addr7527==Address)	   /* AA			 */
      {
      if(HostWdtModuleStatus)	   /* Host Failure		 */
	 sprintf(Echo485,"!");
      else if(Cmd485[4]=='1')      /* Write DO1                  */
	{
	if (Cmd485[5]=='1')        /* turn DO1 ON                */
	   {
	   SetDo1On();
	   onbdo=16;
	   sprintf(Echo485,"!%02X",addr7527);
	   }
	else if (Cmd485[5]=='0')   /* turn DO1 OFF               */
	   {
	   SetDo1Off();
	   onbdo=0;
	   sprintf(Echo485,"!%02X",addr7527);
	   }
	else			   /* invalid command		 */
	   sprintf(Echo485,"?%02X",addr7527);
	}
      }
  else
      sprintf(Echo485,"?%02X",addr7527);
  EchoType=ECHO2_ECHO;
}

/* --- synchronoized sampling ---------------------------------- */

void SyncSample(void) /* valid address: all AA			 */
{
SyncData=GetDi1(); /* DI1=!ReadInitPin() */
FirstRead=1;
}

/* --- Host is OK ---------------------------------------------- */

void HostOk(void) /* valid address: all AA			 */
{
_asm cli
  HostTimerStart=*timetick;
_asm sti
}

/* --- Set Power On Value of DO -------------------------------- */
void SetPowerOnValue(void) /* valid address: AA 		 */
{
HostWdtPowerOnValue=onbdo;
sprintf(Echo485,"!%02X%X",Address,HostWdtPowerOnValue);
EnableEEP();
WriteEEP(7,23,HostWdtPowerOnValue);
ProtectEEP();
}

/* --- Set Safe Value	  of DO -------------------------------- */

void SetSafeValue(void) /* valid address: AA			 */
{
HostWdtSafeValue=onbdo;
sprintf(Echo485,"!%02X%X",Address,HostWdtSafeValue);
EnableEEP();
WriteEEP(7,24,HostWdtSafeValue);
ProtectEEP();
}

/* --- Set all Do ---------------------------------------------- */

void SetAllDo(char);
void SetAllDo(char DoVal) /* valid address: AA			 */
{
onbdo=DoVal&0x10;
if (onbdo) SetDo1On(); else SetDo1Off();


}

/* ------------------------------------------------------------- */

int ReadBuf(void)
{
   int length=0;

   if(addr7527==Address){
      length=GetDataInRingBuf1(Echo485);
      WaitCom1Echo=0;
   }
   else if(addr7527==Address+1){
      length=GetDataInRingBuf_3(Echo485);
      WaitCom1Echo_3=0;
   }
   else if(addr7527==Address+2){
      length=GetDataInRingBuf_4(Echo485);
      WaitCom1Echo_4=0;
   }
   else if(addr7527==Address+3){
      length=GetDataInRingBuf_5(Echo485);
      WaitCom1Echo_5=0;
   }
   else if(addr7527==Address+4){
      length=GetDataInRingBuf_6(Echo485);
      WaitCom1Echo_6=0;
   }
   else if(addr7527==Address+5){
      length=GetDataInRingBuf_7(Echo485);
      WaitCom1Echo_7=0;
   }
   else if(addr7527==Address+6){
      length=GetDataInRingBuf_8(Echo485);
      WaitCom1Echo_8=0;
   }
   else return 0;

   EchoType=ECHO2_ECHO;
   return length;


}

void ByPassTo232(void)
{
//   int length=0;
   int h;
/*
    if(DEBUG_7527)
    {
     int q;
     dPrint(" BypassTo232{");
     for(q=3;q<Idx485;q++)
     {
	if(Cmd485[q]>31)
	  dPrint("%c",Cmd485[q]);
	else dPrint("[%d]",Cmd485[q]);
     }
     dPrint("}");
    }
/*
   for(h=3;h<Idx485;h++)
     {
	if(Cmd485[h]>31)
	  printCom1("%c",Cmd485[h]);
	else printCom1("[%d]",Cmd485[h]);
     }
*/
    WaitCom1Echo=1;
    ResetRingBuf1();
    ToComBufn232(Cmd485+3,Idx485-3); /* send to COM1	  */
//    Show5DigitLed(5,Idx485-3);			  /* for user	       */

    switch(ModeCrLf232){
      case 0:
	   ToCom232Str("\r");        /* CrLfMode=0 --> CR    */
	   break;
      case 1:
	   ToCom232Str("\r\n");      /* CrLfMode=1 --> CR+LF */
	   break;
      case 2:
	   ToCom232Str("\n");        /* CrLfMode=2 --> LF    */
	   break;
      case 3:
	   ToCom232Str("\n\r");      /* CrLfMode=3 --> LF+CR */
	   break;
    }


}

void ByPassTo_3(void)
{
    WaitCom1Echo_3=1;
    ResetRingBuf_3();
    ToComBufn_3(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_3){
      case 0:
	 ToComStr_3("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_3("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_3("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_3("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }

}

void ByPassTo_4(void)
{
    WaitCom1Echo_4=1;
    ResetRingBuf_4();
    ToComBufn_4(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_4){
      case 0:
	 ToComStr_4("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_4("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_4("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_4("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }
}

void ByPassTo_5(void)
{
    WaitCom1Echo_5=1;
    ResetRingBuf_5();
    ToComBufn_5(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_5){
      case 0:
	 ToComStr_5("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_5("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_5("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_5("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }
}

void ByPassTo_6(void)
{
    WaitCom1Echo_6=1;
    ResetRingBuf_6();
    ToComBufn_6(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_6){
      case 0:
	 ToComStr_6("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_6("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_6("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_6("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }
}

void ByPassTo_7(void)
{
    WaitCom1Echo_7=1;
    ResetRingBuf_7();
    ToComBufn_7(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_7){
      case 0:
	 ToComStr_7("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_7("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_7("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_7("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }
}


void ByPassTo_8(void)
{
    WaitCom1Echo_8=1;
    ResetRingBuf_8();
    ToComBufn_8(Cmd485+3,Idx485-3);/* send to COM3	    */
    switch(ModeCrLf_5){
      case 0:
	 ToComStr_8("\r");       /* CrLfMode=0 --> CR    */
	 break;
      case 1:
	 ToComStr_8("\r\n");     /* CrLfMode=1 --> CR+LF */
	 break;
      case 2:
	 ToComStr_8("\n");       /* CrLfMode=2 --> LF    */
	 break;
      case 3:
	 ToComStr_8("\n\r");     /* CrLfMode=3 --> LF+CR */
	 break;
    }
}

/* ------------------------------------------------------------- */

void ProcessCom485(void)
{
 int i;
 int length=0;
 Idx485cmd=0;
 for(i=0;i<Idx485;i++)
    if(Cmd485[i]>31) ++Idx485cmd;//calculate non-control characters
 if(DEBUG_7527)
    dPrint(" Idx485=%d,Idx485cmd=%d",Idx485,Idx485cmd);

// InitPin=ReadInitPin();  /* 1: Init* connect to GND	      */
/* step 1 : process this command string 		      */
 EchoType=NO_ECHO;
 if((Cmd485[0]=='#')&&(Cmd485[1]=='*')&&(Cmd485[2]=='*')&&(Idx485cmd==3))
  SyncSample();
 else if((Cmd485[0]=='~')&&(Cmd485[1]=='*')&&(Cmd485[2]=='*')&&(Idx485cmd==3))
  HostOk();
 else if(Idx485cmd>=3){
  addr7527=(ascii_to_hex(Cmd485[1])<<4)+ascii_to_hex(Cmd485[2]);
  if(addr7527==Address
    || (addr7527==Address+1)
    || (addr7527==Address+2)
    || (addr7527==Address+3)
    || (addr7527==Address+4)
    || (addr7527==Address+5)
    || (addr7527==Address+6)    )
  {

/* ----------- command type 1 --> $???? for 7188XC standard command --- */
     if(Cmd485[0]=='$'){
	COMMAND=1;
	switch(Cmd485[3]){
	  case 'M': /* $AAM: read module name                           */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X7527",Address);
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case 'F': /* $AAF: read firmware version number               */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02XA1.0",Address);/* A1.0        */
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '6': /* $AA6(ID): Set ID, max. length=50 chars           */
	       if(Idx485cmd>4) SetID();
	       break;
	  case '7': /* $AA7: Read ID                                    */
	       if(Idx485cmd==4) ReadID();
	       break;
	  case 'A': /* $AAA[addr]: Read/Set new address                 */
	       if(Idx485cmd==4) ReadAddress();	   /* read AA		*/
	       else if(Idx485cmd==6) SetAddress();    /* set  AA		*/
	       break;
	  case 'B': /* $AABN[baudrate]: Read/Set new baudrate           */
	       if(Idx485cmd==5) ReadComBaudrate();    /* read baudrate	*/
	       else if(Idx485cmd>5) SetComBaudrate(); /* set		*/
	       break;
	  case 'C': /* $AAC[delimiter]: Read/Set new delimiter          */
	       if(Idx485cmd==4){ /* $AAC: Read Delimiter			*/
		 sprintf(Echo485,"!%02X%c",addr7527,Delimiter);
		 EchoType=ECHO2_ECHO;
	       }
	       else /* $AAC[delimiter]: Set new Delimiter		*/
		    if(Idx485cmd==5 && Cmd485[4]!='$') SetDelimiter();
	       break;
	  case 'D': /* $AADN[data-bit] & $AAD                           */
	       if(Idx485cmd==4) ReadDelimiter();	/* $AAD: read delimiter */
	       else if(Idx485cmd==5) ReadDataBit();/* $AADN: read data-bit */
	       else if(Idx485cmd>5) SetDataBit();	/* $AADN[data-bit]: set */
	       break;
	  case 'K': /* $AAK[chksum]: Read/Set checksum mode             */
	       if(Idx485cmd==4) ReadCheckSumMode();
	       else if(Idx485cmd==5) SetCheckSumMode();
	       break;
	  case 'T': /* $AATN[CrLfMode]: Read/Set new CrLfMode           */
	       if(Idx485cmd==5) ReadCRLF();	/* read CrLfMode	*/
	       else if(Idx485cmd>5) SetCRLF();	/* set CrlFMode 	*/
	       break;
	  case 'P': /* $AAPN[Parity-bit]: Read/Set new parity-bit       */
	       if(Idx485cmd==5) ReadParity();	/* read parity-bit	*/
	       else if(Idx485cmd>5) SetParity();	/* set parity-bit	*/
	       break;
	  case 'O': /* $AAON[stop-bit]: Read/Set new stop-bit           */
	       if(Idx485cmd==5) ReadStopBit();	/* read stop-bit	*/
	       else if(Idx485cmd>5) SetStopBit();	/* set stop-bit 	*/
	       break;
	  case 'U': /* $AAW: Read buffer of 232 port                  */
	       if(Idx485cmd==4) length=ReadBuf();
	       break;
	  case 'W': /* $AAW: Read CTS of COM1 or COM3                   */
	       if(Idx485cmd==4) ReadCts();
	       break;
	  case 'X': /* $AAXN: Set RTS of COM1 or COM3                   */
	       if(Idx485cmd==5) SetRts();
	       break;
	  case 'Y': /* $AAYN: Read onboard DI1/2/3 for N=1/2/3          */
	       if(Idx485cmd==5) ReadOnBoardDi();
	       break;
	  case 'Z': /* $AAZNV: Write onboard DO1/2/3 for N=1/2/3,V=0/1  */
	       if(Idx485cmd==6) WriteOnBoardDo();
	       break;
	  case '4': /* $AA4: read the sync-sample data                  */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X%d%X",Address,FirstRead,SyncData);
		 FirstRead=0;
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '5': /* $AA5: read the reset-status                      */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X%d",Address,ResetStatus);
		 ResetStatus=0;
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '2': /* $AA2: read the configuration saved in EEPROM     */
	       if(Idx485cmd==4) ReadConfiguration();
	       break;
	}
     }
/* ----------- command type 2 --> bypass command-string to COM1/3/4 --- */
     else if(Cmd485[0]==Delimiter && addr7527==Address){
	     ByPassTo232();  /* bypass command-string from COM2 to COM1 */
     }
     else if(Cmd485[0]==Delimiter_3 && addr7527==Address+1){
	     ByPassTo_3();/* bypass command-string from COM2 to COM3 */
     }
     else if(Cmd485[0]==Delimiter_4 && addr7527==Address+2){
	     ByPassTo_4();/* bypass command-string from COM2 to COM4 */
     }
     else if(Cmd485[0]==Delimiter_5 && addr7527==Address+3){
	     ByPassTo_5();/* bypass command-string from COM2 to COM5 */
     }
     else if(Cmd485[0]==Delimiter_6 && addr7527==Address+4){
	     ByPassTo_6();/* bypass command-string from COM2 to COM6 */
     }
     else if(Cmd485[0]==Delimiter_7 && addr7527==Address+5){
	     ByPassTo_7();/* bypass command-string from COM2 to COM7 */
     }
     else if(Cmd485[0]==Delimiter_8 && addr7527==Address+6){
	     ByPassTo_8();/* bypass command-string from COM2 to COM8 */
     }
/* ----------- command type 3 --> ~???? for dual-watchdog command ----- */
     else if(Cmd485[0]=='~'){
	COMMAND=1;

	switch(Cmd485[3]){
	  case '0': /* ~AA0: read module status, 7000 compatible        */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X%d",Address,HostWdtModuleStatus);
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '1': /* ~AA1: reset module status, 7000 compatible       */
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X",Address);
		 HostWdtModuleStatus=0;
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '2': /* ~AA2: read host watchdog status & value          */
		    /* 7000 compatible					*/
	       if(Idx485cmd==4){
		 sprintf(Echo485,"!%02X%d%02X",Address,HostWdtEnable,HostWdtTimer&0xff);
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '3': /* ~AA3ETT: enable host watchdog timer              */
		    /* 7000 compatible (partial)			*/
	       if(Idx485cmd==7){
		 if (Cmd485[4]=='1') HostWdtEnable=1; else HostWdtEnable=0;
		 HostWdtTimer=(ascii_to_hex(Cmd485[5])<<4)+ascii_to_hex(Cmd485[6]);
		 sprintf(Echo485,"!%02X%X%02X",Address,HostWdtEnable,HostWdtTimer);
		 EnableEEP();
		 WriteEEP(7,21,HostWdtEnable);
		 WriteEEP(7,22,HostWdtTimer);
		 ProtectEEP();
		 if (HostWdtEnable)
		    {
		    _asm cli
		    HostTimerStart=*timetick;
		    _asm sti
		    }
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '4': /* ~AA4P/S: read power-on/safe value of D/O         */
		    /* 7000 compatible					*/
	       if(Idx485cmd==5){
		 if (Cmd485[4]=='P') sprintf(Echo485,"!%02X%X",Address,HostWdtPowerOnValue);
		 else if (Cmd485[4]=='S') sprintf(Echo485,"!%02X%X",Address,HostWdtSafeValue);
		 else sprintf(Echo485,"?%02X",Address);
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	  case '5': /* ~AA5P/S: Set  power-on/safe value of D/O         */
		    /* 7000 compatible (partial)			*/
	       if(Idx485cmd==5){
		 if (Cmd485[4]=='P') SetPowerOnValue();
		 else if (Cmd485[4]=='S') SetSafeValue();
		 else sprintf(Echo485,"?%02X",Address);
		 EchoType=ECHO2_ECHO;
	       }
	       break;
	}
     }
/* ----------- command type 4 --> @???? for user defined command ------ */
     else if(Cmd485[0]=='@'){
     }
/* ----------- command type 5 --> #???? for user defined command ------ */
     else if(Cmd485[0]=='#'){
     }
/* ----------- command type 6 --> %???? for user defined command ------ */
     else if(Cmd485[0]=='%'){
     }
  }
 }

/* step 2 : ECHO response string to COM2		      */

  i=strlen(Echo485);
  switch(EchoType){
    case NO_ECHO:	  /* 0: NoEcho			      */
	 break;
    case ERROR_ECHO:	  /* 1: Call EchoErrorCmd()	      */
	 EchoErrorCmd();
	 break;
    case OK_ECHO:	  /* 2: Call EchoOkCmd()	      */
	 EchoOkCmd();
	 break;
    case ECHO2_ECHO:	  /* 3: echo string stored in Echo485 */
	 SendCmdToCom485Str(Echo485,(i>length)?i:length);
	 if(DEBUG_7527)
	 {
	    int k,h;
	    h=(i>length)?i:length;
	    dPrint(" ");
	    for(k=0;k<h;k++)
	       if(Echo485[k]>31)
		  dPrint("%c",Echo485[k]);
	       else dPrint("[%d]",Echo485[k]);
	 }
	 if(quit) WaitTransmitOver485();
	 break;
  }

 if(WaitToDo){
    if(WaitToDo==_ChangeCheckSum) { CheckSum=NewCheckSum; }
    else {
       WaitTransmitOver485();
       RestoreCom485();
       InstallCom485(BAUD[Baud485],Data485,Parity485);
    }
    WaitToDo=0;
 }
 Idx485=0;
 Idx485cmd=0;
}

/* --- if comand-format is match then process this command -------- */

void DoCom485(void) /* COM2=RS-485 */
{ int data;
  int Do485=0;
  static unsigned long LastHitTimeTick;

  while(IsCom485()){
    Cmd485[Idx485++]=data=ReadCom485();
    LastHitTimeTick= * timetick;

    if(DEBUG_7527){
       if(data<32) dPrint("[%d]",data);
       else dPrint("%c",data);
    }

    if(data=='\r'){
       switch(ModeCrLf485){
	 case 0:		 /* CrLfMode=0=CR	 */
	      Cmd485[Idx485-1]=0;
//	      if(DEBUG_7527) dPrint("[CR]\n");
	      Idx485--;
	      Do485=1;
	      goto End485;
	 case 3:		 /* CrLfMode=3=LF+CR	 */
	      if(Cmd485[Idx485-2]=='\n'){

		 Cmd485[Idx485-2]=0;
		 Idx485-=2;
//		 if(DEBUG_7527) dPrint("[LF+CR]\n");
		 Do485=1;
		 goto End485;
	      }
	      break;
       }
    }
    else if(data=='\n'){
       switch(ModeCrLf485){
	 case 1: /* CR+LF */	 /* CrLfMode=1=CR+LF	 */
	      if(Cmd485[Idx485-2]=='\r'){

		 Cmd485[Idx485-2]=0;
		 Idx485-=2;
//		 if(DEBUG_7527) dPrint("[CR+LF]\n");
		 Do485=1;
		 goto End485;
	      }
	      break;
	 case 2: /* LF */	 /* CrLfMode=2=LF	 */
	      Cmd485[Idx485-1]=0;
//	      if(DEBUG_7527) dPrint("[LF]\n");
	      Idx485--;
	      Do485=1;
	      goto End485;
       }
    }
  }
////////
  if( (ModeCrLf485==4) && (Idx485!=0))
  {
     if( ((*timetick)-LastHitTimeTick)>IdleTime485)
     {
	   //length=Idx485
//         Cmd485[Idx485]=??-->0~Idx485-1
//	   Cmd485[Idx485]=0;//////////// the string end can not contain 0x00,this line must remove.
	   //Cmd485[Idx485] contains usefull data
//	   Idx485;
	   Do485=1;
     }

  }


///////
End485:
  if(Do485){

    if(DEBUG_7527)
    {
     int q;
     dPrint(" DoCom485{");
     for(q=0;q<Idx485;q++)
     {
	if(Cmd485[q]>31)
	  dPrint("%c",Cmd485[q]);
	else dPrint("[%d]",Cmd485[q]);
     }
     dPrint("}");
    }

     if(CheckSum)
     {
       unsigned char sum=0;
       int i;
       for(i=0;i<Idx485-2;i++){
	   sum+=Cmd485[i];
       }
       if(DEBUG_7527) dPrint("[%02X]",sum);
       if( (hex_to_ascii[sum >> 4] == Cmd485[i])
	  && (hex_to_ascii[sum & 0x0F] == Cmd485[i+1]) ) {
	  Idx485-=2;
//	  Cmd485[Idx485]=0;
       }
       else {	/* checksum is enable & echeck error */
	  Idx485=0;
	  if(DEBUG_7527) dPrint("(Checksum error!)",sum);
	  return;
       }
     }
     ProcessCom485();	 /* process this command */
  }
}

/* ------------------------------------------------------------- */

int  init_user(void);
void main(int argc,char *argv[])
{ int i;
  unsigned long NowTimeTick;

  if(!Is7188xb()){    /* run this program in PC will halt that PC */
     Print("This program must be run under 7188XB/7524/7527.");
     return;
  }

  for(i=1;i<argc;i++) {
      if(argv[i][0]=='/'){
	 switch(argv[i][1]){
	   case 'd': case 'D':
		DEBUG_7527=1;
		/*
		  DEBUG_7522=1 --> go to debug mode, take care as following:
		  note 1: COM1 is used to show debug message
		  note 2: do not call InstallCom1()
		  note 3: and do not echo COM1 message back to COM2
		*/
		break;

	 }
      }
  }




  init_user();
  if(DEBUG_7527) dPrint("Start 7527!\n\r");

  Open7527();
  EnableWDT();
  while(!quit){
    RefreshWDT();
    if(!DEBUG_7527) DoCom232();   /* if receive string from COM1 --> send to COM2 */
    DoCom_3(); /* if receive string from COM3 --> send to COM2 */
    DoCom_4(); /* if receive string from COM4 --> send to COM4 */
    DoCom_5(); /* if receive string from COM5 --> send to COM5 */
    DoCom_6(); /* if receive string from COM6 --> send to COM6 */
    DoCom_7(); /* if receive string from COM7 --> send to COM7 */
    DoCom_8(); /* if receive string from COM8 --> send to COM8 */

    DoCom485();   /* process command from COM2			  */

    if (HostWdtEnable) {
       _asm cli
       NowTimeTick=*timetick;
       _asm sti
       if (NowTimeTick>=HostTimerStart) i=NowTimeTick-HostTimerStart;
       else i=0xffffffffffffffff-HostTimerStart+NowTimeTick;
       if (i>(HostWdtTimer*100)) {
	  HostWdtModuleStatus=0x04;
	  SetAllDo(HostWdtSafeValue);
	  HostWdtEnable=0;
       }
    }

    if(DEBUG_7527){
      if(IsComd() && ReadComd()=='q') quit=1;
    }
  }
  DisableWDT();
  Close7527();
}


int init_user(void)
{
   return 0;
}
